﻿--[[
Changelog:
    Version 1.0.4
        Updated the crit surpression from 1% per level to 1.5% per level

    Version 1.0.3
        Updated the rage gained from auto attacks
        Attempt to factor in the chance to proc mangle of cooldown

    Version 1.0.2
        Updated TOC version number

	Version 1.0.1
		Added crit reduction to the rage per second calculation

	Version 1.0.0
		First working version
		
Known Bugs:
	None

Todo:
	Factor in chance to proc mangle of cooldown
--]]

--main global var
local RagePerSecond = {
	["NAME"] 		= "|TInterface\\Icons\\Ability_Druid_Enrage:0|t |cFFCC0033Rage Per Second|r",
	["VERSION"] 	= "(|cFF9900001|cFFFFFFFF.|cFF9900000|cFFFFFFFF.|cFF9900004|cFFFFFFFF)|r",
}
--

--event array
local RagePerSecondEventArray = {}
function RagePerSecond_Event(self,event,...) RagePerSecondEventArray[event](self,...) end
--

--local constant
local NORMALFORMAT = "%.02f"
local GREENFORMAT = GREEN_FONT_COLOR_CODE .. NORMALFORMAT .. FONT_COLOR_CODE_CLOSE
local PLUSGREENFORMAT = GREEN_FONT_COLOR_CODE .."+".. NORMALFORMAT .. FONT_COLOR_CODE_CLOSE
--

-------------------
--Other Functions--
-------------------
local function GetSavageDefenceRageCap()
    local _, rageCost
    _, _, _, rageCost = GetSpellInfo(62606) --Savage Defense
    
    return (rageCost or 60) / 9
end

local function PlayerHasEnrageBuff()
    --Druid Enrage
    return UnitBuff("player", GetSpellInfo(5229), "", "PLAYER|CANCELABLE") ~= nil
end

--0.23701331018518 = mangles per second
--10.9375 = rage from a single auto attack
local function GetRagePerSecond(levelOffset)
    local mangleBaseRage
    if Silvanas.Select(5, GetTalentInfo(10)) then --player has SotF
        mangleBaseRage = 7
    else
        mangleBaseRage = 5
    end
    
    local chanceToHit = (100 - GetEnemyDodgeChance(levelOffset) - GetEnemyParryChance(levelOffset) - GetMeleeMissChance(levelOffset)) / 100
    local rageFromCrit = 15 * ((GetCritChance() - (levelOffset * 1.5)) / 100)
    local rageFromMelee = ((10.9375 + rageFromCrit) / UnitAttackSpeed("player")) * chanceToHit
    local rageFromMangle = (mangleBaseRage + rageFromCrit) * 0.23701331018518 * chanceToHit
    
    return rageFromMelee + rageFromMangle, rageFromMelee, rageFromMangle
end

local function RagePerSecond_OnEnter(self)
    if MOVING_STAT_CATEGORY then return end

    local enraged = PlayerHasEnrageBuff()
    local totalRagePerSecond, rageFromMelee, rageFromMangle = GetRagePerSecond(0)

    GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
    
    if enraged then
        GameTooltip:AddLine(string.format("Rage Per Second %.02f (%.02f".. PLUSGREENFORMAT ..")", totalRagePerSecond + 1, totalRagePerSecond, 1), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
        totalRagePerSecond = totalRagePerSecond + 1
    else
        GameTooltip:AddLine(string.format("Rage Per Second %.02f", totalRagePerSecond), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
    end
    
    GameTooltip:AddLine(string.format("|cFF888888(Savage Defense softcap = %.02f)|r", GetSavageDefenceRageCap()))
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine("Ability", "Rage Per Second", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	GameTooltip:AddDoubleLine("      Melee", string.format("%.02f", rageFromMelee) .."    ", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
   	GameTooltip:AddDoubleLine("      Mangle", string.format("%.02f", rageFromMangle) .."    ", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
    
    local percentText 
    if enraged then 
        formatText = GREENFORMAT
        GameTooltip:AddDoubleLine("      Enrage", string.format(formatText, 1) .."    ", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
    else 
        formatText = NORMALFORMAT
    end
    
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine(STAT_TARGET_LEVEL, "Rage Per Second", HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)

    local unitLevel = UnitLevel("player")
    local i = 0
    repeat
        local level = unitLevel + i
        if (i == 3) then
			level = level .." / |TInterface\\TargetingFrame\\UI-TargetingFrame-Skull:0|t"
		end
		GameTooltip:AddDoubleLine("      ".. level, string.format(formatText, totalRagePerSecond) .."    ", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
        
        i = i + 1
        totalRagePerSecond = GetRagePerSecond(i)
        
        if enraged then
            totalRagePerSecond = totalRagePerSecond + 1
        end        
	until i > 3

    GameTooltip:Show()
end

local function Armor_OnEnter(self)
    if MOVING_STAT_CATEGORY then return end
    
    local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor(self._Unit)

    GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
    GameTooltip:AddLine(self._OldArmorToolTip, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine(STAT_TARGET_LEVEL, COMBAT_TEXT_SHOW_RESISTANCES_TEXT, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)

    local unitLevel = UnitLevel(self._Unit)
    for i=0, 3 do
        local level = unitLevel + i
		local armorReduction = string.format("%.2f%%", PaperDollFrame_GetArmorReduction(effectiveArmor, level))
        if (i == 3) then
			level = level .." / |TInterface\\TargetingFrame\\UI-TargetingFrame-Skull:0|t"
		end
		GameTooltip:AddDoubleLine("      ".. level, armorReduction .."    ", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	end

    if self.Unit == "player" then
		local petBonus = ComputePetBonus("PET_BONUS_ARMOR", effectiveArmor)
		if petBonus > 0 then
            GameTooltip:AddLine(" ")
            GameTooltip:AddLine(string.format(PET_BONUS_TOOLTIP_ARMOR, petBonus))
		end
	end
   
    GameTooltip:Show()
end
-------------------
-------------------
-------------------

-------------------
--Event Triggered--
-------------------
--onload
function RagePerSecond_Load(self)
    --hooking the update function
    hooksecurefunc(PAPERDOLL_STATINFO["POWER"], "updateFunc",
        function(statFrame, unit)
            unit = unit or "player" 
            
            powerFrame = statFrame
            
            local powerType = UnitPowerType(unit);
            if powerType ~= 1 then --1 = Rage
                return
            end
            
            statFrame.tooltip = nil
            statFrame.tooltip2 = nil
            
            statFrame:HookScript("OnEnter", RagePerSecond_OnEnter)
            
            local totalRagePerSecond = GetRagePerSecond(0)
            local formatText = NORMALFORMAT

            if PlayerHasEnrageBuff() then
                totalRagePerSecond = totalRagePerSecond + 1
                formatText = GREENFORMAT
            end
            
            PaperDollFrame_SetLabelAndText(statFrame, "RPS", string.format(formatText, totalRagePerSecond))
        end
    )
    
    hooksecurefunc(PAPERDOLL_STATINFO["ARMOR"], "updateFunc",
        function(statFrame, unit)
            unit = unit or "player" 

            statFrame._OldArmorToolTip = statFrame.tooltip --dont need to waste a perfect tooltip line
            statFrame._Unit = unit    
            
            statFrame.tooltip = nil
            statFrame.tooltip2 = nil
            
            statFrame:HookScript("OnEnter", Armor_OnEnter)
        end
    )
    
	--loaded message
	Silvanas.Print(RagePerSecond["NAME"] .." |cFFFFFFFFBy: ".. Silvanas.Author .." |cFFFFFFFFversion: ".. RagePerSecond["VERSION"] .." |cFFFFFFFFloaded.|r")

end
-------------------
-------------------
-------------------